import os
import sys
import time
import zmq
import logging
import threading
import cPickle

BASE_PATH   = os.path.dirname ( os.path.dirname ( os.path.dirname ( os.path.realpath ( __file__ ) ) ) )
API_PATH    = BASE_PATH + os.sep + "APIs"
SHARED_PATH = BASE_PATH + os.sep + "src" + os.sep + "shared"

if not API_PATH in sys.path:
    sys.path.append ( API_PATH )
del API_PATH

from nexusTransferAPI import nexusTransfer

if not SHARED_PATH in sys.path:
    sys.path.append ( SHARED_PATH )
del SHARED_PATH

import helpers

#enable logging
logfilePath = os.path.join(BASE_PATH + os.sep + "logs")
logfile     = os.path.join(logfilePath, "testNexusTransferAPI.log")
helpers.initLogging(logfile, True, "DEBUG")


print
print "==== TEST: nexus transfer ===="
print

class Sender(threading.Thread):
    def __init__(self):
        self.extHost    = "0.0.0.0"
        self.localhost  = "localhost"
        self.signalPort = "50050"
        self.dataPort   = "50100"

        self.context       = zmq.Context()

        self.signalSocket  = self.context.socket(zmq.REQ)
        connectionStr = "tcp://" + str(self.localhost) + ":" + str(self.signalPort)
        self.signalSocket.connect(connectionStr)
        logging.info("signalSocket started (connect) for '" + connectionStr + "'")

        self.dataSocket    = self.context.socket(zmq.PUSH)
        connectionStr = "tcp://" + str(self.localhost) + ":" + str(self.dataPort)
        self.dataSocket.connect(connectionStr)
        logging.info("dataSocket started (connect) for '" + connectionStr + "'")

        threading.Thread.__init__(self)


    def run(self):
        message = "OPEN_FILE"
        self.signalSocket.send(message)

        recvMessage = self.signalSocket.recv()

        for i in range(5):
            metadata = {
                "sourcePath"  : BASE_PATH + os.sep +"data" + os.sep + "source",
                "relativePath": "local",
                "filename"    : str(i) + ".cbf"
                }
            metadata = cPickle.dumps(metadata)
            data     = "THISISTESTDATA-" + str(i)

            dataMessage = [metadata, data]
            self.dataSocket.send_multipart(dataMessage)
            logging.debug("Send")


    def stop(self):
        try:
            if self.signalSocket:
                logging.info("Closing signalSocket...")
                self.signalSocket.close(linger=0)
                self.signalSocket = None
            if self.dataSocket:
                logging.info("Closing dataSocket...")
                self.dataSocket.close(linger=0)
                self.dataSocket = None
            if self.context:
                logging.info("Destroying context...")
                self.context.destroy()
                self.context = None
                logging.info("Destroying context...done")
        except:
            logging.error("Closing ZMQ Sockets...failed.", exc_info=True)



senderThread = Sender()
senderThread.start()



obj = nexusTransfer(useLog = True)

for i in range(6):
    try:
        data = obj.read()
        logging.debug("Retrieved: " + str(data))
    except:
        logging.error("break", exc_info=True)
        break

senderThread.stop()
obj.stop()

print
print "==== TEST END: nexus transfer ===="
print



